# -*- coding: utf-8
# $Id: GroupFolder.py,v 1.4 2003/08/18 07:44:24 berlingo Exp $
#
# Copyright 2001, 2002 by IVA Team and contributors
#
# This file is part of IVA.
#
# IVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# IVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with IVA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Contains class GroupFolder, which is the root object for group folders
in Knowledge Building."""

__version__ = "$Revision: 1.4 $"[11:-2]

import Globals
#from Globals import Persistent, HTMLFile
import AccessControl

from common import reload_dtml, add_dtml
#import TraversableWrapper
import WebtopFolder
import WebtopTrash
from WebtopItem import WebtopItem
from TraversableWrapper import TWFolder
from TempObjectManager import TempObjectManager
from common import perm_view, perm_edit, perm_manage, perm_add_lo

# This object is the root of a group folder inside a Course or
# a CourseContext.
#
# A group folder contains a trash folder and any
# WebtopItems the users have created. WebtopItem is a superclass, and
# actually the group folder will contain specific subclass instances, like
# WebtopFolder, WebtopFile, WebtopLink and WebtopMemo.
#
class GroupFolder(
    WebtopFolder.WebtopFolder,
    ):
    """Group folder."""
    meta_type = "GroupFolder"

    list_of_types=('WebtopFolder', 'WebtopLink', 'WebtopMemo', 'WebtopFile','GroupFolder')

    security = AccessControl.ClassSecurityInfo()
    def __init__(self,parent,name):
        """Construct the group folder root object."""
        WebtopItem.__init__(self,parent,name)
        TempObjectManager.__init__(self)
        self.id = 'gf' + self.id[2:]
        self.__id_counter = long(self.id[2:])

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Create trash and clipboard and call superclass
        initialization code."""
        WebtopFolder.WebtopFolder.manage_afterAdd(self, item, container)

    security.declarePrivate('generate_id')
    def generate_id(self):
        """Return a unique (within this webtop) id.
        These ids are used for any and all WebtopItems created
        instead of their visible names so we avoid any problems with
        visible name conflicts and can use more complex visible names
        for items."""

        self.__id_counter += 1L
        return "gf"+str(self.__id_counter)

    security.declareProtected(perm_view, 'get_list_item_name')
    def get_list_item_name(self, REQUEST=None):
        """Return the name of Groupfolder."""
        return self.get_name()

    def get_name(self):
        """Return the name of the course."""
        try:
            if hasattr(self.parent().aq_base, 'toplevel'):
                return self.parent().get_name()
        except AttributeError:
            pass
        return WebtopFolder.WebtopFolder.get_name(self)

##     security.declarePrivate('add_folder')
##     def add_folder(self,name):
##         """Implementation of add_folder_handler without http code."""
##         f = GroupFolder(self,name)
##         self._setObject(f.id,f)
##         return f

    security.declarePrivate('get_clipboard')
    def get_clipboard(self):
        uname=str(self.REQUEST.AUTHENTICATED_USER)
        return self.fle_users.get_user_info(uname).webtop.clipboard

    security.declareProtected(perm_view, 'get_trash')
    def get_trash(self):
        """Return trash (an instance of WebtopTrash)"""
        #raise 'foo',str(self.aq_chain)
        if hasattr(self.parent().aq_base, 'toplevel'):
            return self.get_child('trash')
        else:
            return self.parent().get_trash()

    def kysiSyndmusteKataloog(self):
      "Lisatakse kui pole"
      if self.absolute_url().find('subgroups')==-1:
        return self.aq_parent.kysiSyndmusteKataloog()
      import Kalender
      if not hasattr(self, 'syndmused'):
        self._setObject('syndmused', Kalender.KalendriSyndmusteKataloog())
        self.syndmused.id='syndmused'
      return self.syndmused

Globals.InitializeClass(GroupFolder)

# EOF
