# -*- coding: utf-8
# Copyright 2001, 2002 by IVA Team and contributors
#
# This file is part of IVA.
#
# IVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# IVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with IVA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import AccessControl
import OFS, Globals
from OFS import SimpleItem
import types
import time
from TraversableWrapper import Traversable
from Globals import Persistent
from AccessControl import ClassSecurityInfo
from Cruft import Cruft
from common import add_dtml, translate

class KalendriSyndmus(
     OFS.SimpleItem.Item,
     Persistent,
     AccessControl.Role.RoleManager,
     Traversable,
     Cruft
     ):
     """ Teade kalendris """
     meta_type = 'Event'
     security = ClassSecurityInfo()
     security.declareObjectPublic()
     
     def __init__(self, nimi, lyhi='', pikk='', algus=-1, lopp=-1, viide="", siseviide="", olek=0,blokerid=[]):
       "Alustus"
       if algus == -1:
	algus = time.time()
       if lopp == -1:
	lopp = time.time()
       self.nimi=nimi
       self.lyhi=lyhi
       self.pikk=pikk
       self.algus=algus
       self.lopp=lopp
       self.viide=viide
       self.siseviide=siseviide
       self.blokerid = []
       self.olek=olek #0-ajatu sündmus 1-lõpetamata töö 2-lõpetatud töö
       self.icon_tag=None
       
     def kysiNimi(self):
       "Teate nimi"
       return self.nimi

     def getIconTag(self):
         if not hasattr(self,'icon_tag'):
		 self.icon_tag = None
         if self.icon_tag is not None:
		 result = "<img src=\""+self.icon_tag+"\" alt=\"This is homework event\" title=\"This is homework event\" />"
         else:
		 result = ""
         return result

     def setIconTag(self,path):
         self.icon_tag = path
         self.icon_tag = self.icon_tag
	 return self.icon_tag
       
     def kysiLyhi(self):
       "Lühikirjeldus"
       return self.lyhi
       
     def kysiPikk(self):
       "Pikk kirjeldus"
       return self.pikk
       
     security.declarePublic('kysiAlgus')
     def kysiAlgus(self):
       "Syndmuse algusaeg"
       return self.algus
       
     def kysiLopp(self):
       "Syndmuse lõppaeg"
       return self.lopp
       
     def aegTekstina(self, REQUEST, aeg, timestamp=''):
       "tekstiesitus"
       if aeg==-1: return "*"
       if not timestamp:
	       timestamp = translate(self,'timestamp_format',target=self.giveLanguage(REQUEST))
       else:
	       timestamp = REQUEST[timestamp]
       return time.strftime(timestamp, time.localtime(aeg))

     def tekstAjana(self, REQUEST, ajatekst):
       "Tagasi ajaformaati"
       return mktime(time.strptime(translate(self,'timestamp_format',target=self.giveLanguage(REQUEST))), ajatekst)

     def kysiViide(self):
       "Välisviide"
       return self.viide
       
     def kysiSiseviide(self):
       "Siseviide"
       return self.siseviide
       
     def kysiSiseviiteURL(self):
       "Siseviide"
       return self.fle_root().absolute_url()+self.siseviide
       
     def siseviiteNimi(self):
       "Nagu funktsiooni nime ütleb"
       import re
       if self.siseviide=="": return ""
       m=self.siseviide.split("/")
       o=self.fle_root()
       try:
         for x in m:
           if len(x)>0:
            o=getattr(o, re.sub('%20',' ',x))
       except:
         return ""
       return o.get_name()
       #return getattr(self.fle_root(), self.siseviide)
       #return str(self.get_object_of_url(self.siseviide, self.fle_root()))
       
     def kysiSiseviiteLink(self):
       "Koos"
       return "<a href='"+self.kysiSiseviiteURL()+"'>"+self.siseviiteNimi()+"</a>"
       
     def kysiOlek(self):
       "Sündmuse olek"
       if hasattr(self, 'olek'): return self.olek
       return 0

     def getPersonalStatus(self,REQUEST):
	 """ """
	 if hasattr(self, 'status_'+str(REQUEST.AUTHENTICATED_USER)):
		 return getattr(self,'status_'+str(REQUEST.AUTHENTICATED_USER))
	 else:
		 return 0
	 return 0
       
     def olekuTekst(self, REQUEST, nr=-1):
       "Olek tekstina"
       if nr==-1: nr=self.kysiOlek()
       if nr==0: return ""
       if nr==1: return "incomplete"
       if nr==2: return "complete"
       
     def olekuLahter(self, REQUEST):
       "Olekut illustreeriv lahter"
       if self.kysiOlek()==0:
         return "<td>&nbsp;</td>"
       if self.kysiOlek()==1: #pooleli
         if time.time()>self.lopp: #Hilinenud
           return "<td bgcolor='red'>&nbsp;</td>"
         else: return   "<td bgcolor='blue'>&nbsp;</td>"
       return  "<td bgcolor='green'>&nbsp;</td>"       
       
     def olekuVarv(self, REQUEST):
       "Olekut illustreeriv lahter"
       if self.getPersonalStatus(REQUEST):
	       stat = self.getPersonalStatus(REQUEST)
       else:
	       stat = self.kysiOlek()
       if stat==0:
         return "\"black\""
       if stat==1: #pooleli
         if time.time()>self.lopp: #Hilinenud
           return "\"red\""
         else: return   "\"blue\""
       return  "\"green\""       
       
       
     def tabeliRida(self, REQUEST):
       "Osa tabelist"
       tulemus="<tr>"+\
         self._tabeliReaSisu(REQUEST)+"</tr>\n"
       return tulemus

     def muutmisTabeliRida(self, REQUEST,select=0,omanik=0,timestamp=''):
       "Osa tabelist"
       tulemus="<tr>"
       kast="<td width=\"30\" class=\"t1\">"
       kast += "<input type=\"checkbox\" "
       if select:
	       kast+="checked=\"checked\""
       kast+=" name=\"kustutus\" value=\""+str(self.id)+"\" /></td>"
       if omanik:
	       if self.kysiOmanik().getUserName()==REQUEST.AUTHENTICATED_USER.getUserName() and self.asukohaNimi()==None:
		       tulemus+=kast
	       else:
		       tulemus+="<td width=\"30\" class=\"t1\">"
		       tulemus+="<img src=\"images/transparent_gif\" height=\"8\" width=\"10\" alt=\"\" /></td>"
       else:
	       
	       tulemus+=kast
       tulemus+=self._tabeliReaSisu(REQUEST,timestamp=timestamp)+"</tr>\n"
       return tulemus
       
     def paevikuTabeliRida(self, REQUEST, timestamp=''):
       "Avalehe päevikusse"
       tulemus="<tr>"+self._tabeliReaSisu(REQUEST,timestamp=timestamp)+"</tr>\n"
       return tulemus
       
     def _tabeliReaSisu(self, REQUEST, timestamp=''):
	"Abitükk teistele"
	# FIXFIX: stupid url checking...
	asukoht = REQUEST.URL0.split("/")
	if not asukoht[-1] == "syndmusteMuutmisVorm" and not asukoht[-1] == "kalendriPaevaSyndmusteLoetelu":
		tulemus = "<td width=\"30\" class=\"t1\">"
		tulemus += "<input type=\"checkbox\" value=\""+self.absolute_url()+"\" name=\"synd_id\" />"
		tulemus += "</td>"
	else:
		tulemus = ""
	tulemus += "<td class=\"t1\"><font color=\""+self.olekuVarv(REQUEST)+"\">"+self.asukohtTekstina(REQUEST)+"</font></td><td class=\"t1\" width=\"50\"><font color=\""+self.olekuVarv(REQUEST)+"\">"+\
	      self.aegTekstina(REQUEST, self.kysiLopp(),timestamp=timestamp)+\
              "</font></td><td class=\"t1\"><a href=\""+self.absolute_url()+"\">"+\
              self.getIconTag()+self.kysiNimi()+"</a>" #<td class=\"t1\">"
	tulemus += " <a href=\""+self.absolute_url()+"/vcal\">*</a>"
	tulemus += "</td>"
#              self.aegTekstina(REQUEST, self.kysiAlgus())+\
#       if self.kysiOmanik().getUserName()==REQUEST.AUTHENTICATED_USER.getUserName():
#         tulemus=tulemus+self.kysiLyhi()
#       else:
#         tulemus=tulemus+self.kysiLyhi()+" " #võõras sündmus
#       tulemus=tulemus+"</td>"       
	return tulemus
            
     def asukohaNimi(self):
       "Kursuse nimi, kus sündmus asub, muu sündmuse puhul None"
       aadress=self.absolute_url()
       algus=aadress.find('courses')
       if algus>=0:
         nr=aadress[algus:].split("/")[1]
         t=getattr(self.fle_root().courses, nr).get_name()
         m=aadress[algus:].split("/")
         for i in range(len(m)):
           if m[i]=="subgroups" and i+1<len(m):
             t=t#+" / " + m[i+1]
         return t
       return None
       
     def asukohtTekstina(self, REQUEST):
       "Sõnastatuna tabelisse kandmiseks"
       t=self.asukohaNimi()
       if t is None:
         if REQUEST.AUTHENTICATED_USER==self.kysiOmanik(): 
           return translate(self,'Own',target=self.giveLanguage(REQUEST))
         else:
           return self.kysiOmanik().getUserName()
       return t#+" / " + self.kysiOmanik().getUserName()
            
       
     security.declarePublic('kysiOmanik')
     def kysiOmanik(self):
       "Sündmuse looja"
       return self.getOwner()

     def kasOmanik(self, REQUEST):
       "jah/ei"
       return str(REQUEST.AUTHENTICATED_USER)==str(self.kysiOmanik())
       
     def index_html(self, REQUEST):
       "Väljanäitus"
       return self.fle_root().event_index(REQUEST, event=self)
       tulemus=self.hdr(self, REQUEST)
       tulemus=tulemus+"</p>\n"+self.vaatamisSisu(REQUEST)
       tulemus=tulemus+self.fle_html_footer(self, REQUEST)
       return tulemus

     def userStatus(self,REQUEST,submitt='',olek='',userLocation=''):
	""" User changes event status for himself """
	if not submitt:
		return
	if not olek:
		return
	keyword = 'status_'+str(REQUEST.AUTHENTICATED_USER)
	try:
		setattr(self, keyword,int(olek))
	except:
		return "something happened"
	return REQUEST.RESPONSE.redirect(self.absolute_url()+'?userLocation='+userLocation)
	
     def muutmisLeht(self, REQUEST,userLocation=''):
       "Sündmuse muutus"
       import YlTest
       from YlTest import LahendusLuba
       abi=LahendusLuba() #vaid ajamenüü tarbeks
       self.algus=abi.ajaPyydmisLeht(REQUEST, "algus")
       self.lopp=abi.ajaPyydmisLeht(REQUEST, "lopp")
       if self.lopp<self.algus: self.lopp=self.algus
       self.nimi=REQUEST['nimi']
       self.lyhi=REQUEST['lyhi']
       self.pikk=REQUEST['pikk']
       self.viide=REQUEST['viide']
       self.siseviide=REQUEST['siseviide']
       self.olek=int(REQUEST['olek'])
       REQUEST.RESPONSE.redirect('index_html?userLocation='+userLocation)
       
     def kasSyndmusPaeval(self, aasta, kuu, paev):
       "jah/ei"
       algusm=time.localtime(self.algus)
       algusmp=time.mktime((algusm[0], algusm[1], algusm[2], 0, 0, 0, 0, 0, 1))
       loppm=time.localtime(self.lopp)
       loppmp=time.mktime((loppm[0], loppm[1], loppm[2]+1, 0, 0, 0, 0, 0, 1))
       aegmp=time.mktime((aasta, kuu, paev, 0, 0, 0, 0, 0, 1))
       if aegmp>=algusmp and aegmp<loppmp:return 1
       return 0
       
     def kasLubatudParandada(self, REQUEST):
       "Muutmisõigus"
       if 'Owner' in get_local_roles(self,uname):
           return 1
       if 'Teacher' in get_roles(self,uname):
           return 1
       return 0

     def vcal(self, REQUEST):
	""" Export!? """
	from ImportExportIMS import Kirjutus
	Kir = Kirjutus(self.fle_root(), None)
	kat = 'kursus'
	syndmus = self
	vCal = "BEGIN:VCALENDAR\r\n"
	vCal += "METHOD:PUBLISH\r\n"
	#vCal += "VERSION:2.0\r\n"
	#vCal += "PRODID:IVA\r\n"
	vCal += "BEGIN:VEVENT\r\n"
	vCal += "DTSTART:"+Kir.vCalTime(syndmus.algus)+"\r\n"
	vCal += "DTEND:"+Kir.vCalTime(syndmus.lopp)+"\r\n"
	#vCal += "N:"+omanik+"\r\n"
	vCal += "DESCRIPTION:"+Kir.format(syndmus.pikk)+"\r\n"
	vCal += "SUMMARY:"+Kir.format(syndmus.lyhi)+"\r\n"
	vCal += "STATUS:"+str(syndmus.olek)+"\r\n"
	vCal += "CUTYPE:"+kat+"\r\n"
	vCal += "URL:"+syndmus.viide+"\r\n"
	vCal += "X-PROPERTY;VALUE=NAME:"+Kir.format(syndmus.nimi)+"\r\n"
	vCal += "RESOURCES:"+Kir.format(syndmus.siseviide)+"\r\n"
	vCal += "END:VEVENT\r\n"
	vCal += "END:VCALENDAR\r\n"
	REQUEST.RESPONSE.setHeader('Content-disposition','attachment; filename=event.ics')
	REQUEST.RESPONSE.setHeader('content-type','text/calendar')
	return vCal
       
class KalendriSyndmusteKataloog(
     OFS.Folder.Folder,
     OFS.SimpleItem.Item,
     Persistent,
     AccessControl.Role.RoleManager,
     Traversable,
     Cruft
     ):
     
     syndmuseNr=0
     
     def kysiUusNr(self):
       self.syndmuseNr=self.syndmuseNr+1
       while hasattr(self.aq_self, 'event'+str(self.syndmuseNr)):
	       self.syndmuseNr=self.syndmuseNr+1
       return self.syndmuseNr
     
     def syndmusteTabel(self, REQUEST):
       "Kataloogis olevad sündmused"
       loetelu=self.objectValues()
       tulemus=self.hdr(self, REQUEST)+"<table>"
       for syndmus in loetelu:
         tulemus=tulemus+syndmus.tabeliRida(REQUEST)
       tulemus=tulemus+"</table>"+self.fle_html_footer(self,REQUEST)
       return tulemus
       
     def index_html(self, REQUEST):
       "tabel"
       return self.event_list()
       
       
     def syndmusteMuutmisLeht(self, REQUEST, kustutus=(),backlink='',userLocation="Organizer"):
       "Sündmuste kustutamine"
       if type(kustutus)==types.StringType:
         kustutus=(kustutus,)
       eurl = self.absolute_url()
       for x in kustutus:
           self._delObject(x.split('/')[-1])
       #REQUEST.RESPONSE.redirect('syndmusteMuutmisVorm')
       if backlink:
	       return REQUEST.RESPONSE.redirect(backlink)
       return REQUEST.RESPONSE.redirect(REQUEST.HTTP_REFERER)
#       return self.event_list()
       
     def addEvent(self, REQUEST,userLocation='Organizer'):
       "Uus sündmus"
       nr='event'+str(self.kysiUusNr())
       k=KalendriSyndmus(nr)
       if hasattr(REQUEST, 'etime'):
	       timesta = time.localtime(time.time())
	       t_algus = []
	       t_algus.append(int(REQUEST.etime[:4]))
	       t_algus.append(int(REQUEST.etime[4:6]))
	       t_algus.append(int(REQUEST.etime[6:]))
	       for x in timesta[3:]:
		       t_algus.append(x)
	       k.algus = time.mktime(t_algus)
	       k.lopp = k.algus
       k.id=nr
       self._setObject(nr, k)
#       REQUEST.RESPONSE.redirect(nr+"/muutmisVorm")
       return REQUEST.RESPONSE.redirect(nr+'/changeEvent?userLocation='+userLocation)

     def lisaSyndmusKohe(self, nimi, lyhi, pikk, start, end, url, res, status):
	nr = str(self.kysiUusNr())
	k = KalendriSyndmus(nimi, lyhi, pikk, start, end, url, res, status)
	k.id = nr
	self._setObject(nr, k)
	return k

     def syndmusteKogum(self,REQUEST=None, mida='', sort='', previous='', etime='', private=''):
	 # for use in event_list ZPT
	 return self.objectValues()
       
